'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var http = require('http');
var cheerio = require('cheerio');
var fs = require('fs');
var url = require('url');
var reactRouterDom = require('react-router-dom');
var server = require('react-dom/server');
var path = require('path');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const DEFAULT_WEBPACK_SERVER_PORT = 8080;
const DEFAULT_DEV_SERVER_PORT = 3030;
const DEFAULT_CLIENT_PUBLIC_PATH = '/static/';
const DEFAULT_SERVER_PUBLIC_PATH = '/';
const rootDirectory = process.cwd();
function getDevConfig() {
    return {
        clientChunkName: 'client.js',
        serverChunkName: 'server.[hash:4].js',
        webpackServerPort: DEFAULT_WEBPACK_SERVER_PORT,
        devServerPort: DEFAULT_DEV_SERVER_PORT,
        clientPublicPath: DEFAULT_CLIENT_PUBLIC_PATH,
        serverPublicPath: DEFAULT_SERVER_PUBLIC_PATH,
        htmlPath: path.join(rootDirectory, 'src/index.html'),
    };
}

const devConfig = getDevConfig();
class Server {
    constructor(opts) {
        this.port = 3030;
        this.handleRequest = (req, res) => {
            const routes = this.routes;
            const isMatched = routes.some(route => {
                return !!reactRouterDom.matchPath(req.url, route);
            });
            if (isMatched) {
                const AppContainer = this.AppContainer;
                const content = server.renderToString(React.createElement(reactRouterDom.StaticRouter, { location: req.url, context: {} },
                    React.createElement(AppContainer, null, routes.map((route, index) => {
                        return (React.createElement(reactRouterDom.Route, Object.assign({ key: index }, route)));
                    }))));
                const html = this.renderHTML(content);
                res.setHeader('Content-Type', 'text/html');
                res.end(html);
            }
            else {
                res.end('Page not found');
            }
        };
        this.renderHTML = (content) => {
            const $ = cheerio.load(this.originalHTML);
            $(this.container).append(content);
            $('body').append(`
      <script type='text/javascript' src='${this.clientChunkPath}'></script>
    `);
            return $.html();
        };
        this.clientChunkPath = new url.URL(devConfig.clientChunkName, `http://localhost:${devConfig.webpackServerPort}${devConfig.clientPublicPath}`);
        this.container = opts.container;
        this.AppContainer = opts.AppContainer || React.Fragment;
        this.routes = opts.routes;
        this.originalHTML = fs.readFileSync(devConfig.htmlPath, 'utf-8');
    }
    start() {
        this.app = http.createServer(this.handleRequest);
        this.app.listen(this.port, () => {
            console.log('Server listen on: http://localhost:' + this.port);
        });
    }
    close(a) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.app) {
                yield this.app.close();
            }
        });
    }
}
function render(opts) {
    const server = new Server(opts);
    server.start();
}

exports.render = render;
